#include <stdio.h>
#include <getopt.h>
#include <sys/types.h>

#define LOCAL_TEST

int get_hex(char *p) {
	if ('0' <= *p && *p <= '9') return *p - '0';
	if ('a' <= *p && *p <= 'f') return *p - 'a' + 10;
	if ('A' <= *p && *p <= 'F') return *p - 'A' + 10;
	return 0;
}

int get_data(char *p,int len) {
	int r = 0;
	int i;
	for (i=0; i<len; i++) {
		r <<= 4;
		r |= get_hex(p);
		p++;
	}
	return r;
}

int load_ihex(char *fname, char *data, int size) {
	FILE *fp;
	char buf[256],*p,*out;
	int i,r,ret = 0;
	int len,addr,type,sum,fsum;
	int line_no = 0;
	fp = fopen(fname, "r");
	if (!fp) {
		perror("open");
		return -1;
	}
	sum = 0;
	while (fgets(buf,256,fp)) {
		line_no ++;
		p = buf;
		if (*p != ':') continue;
		p ++;
		len = get_data(p, 2); p += 2;
		sum += len;
		addr = get_data(p, 4); p += 4;
		sum += addr >> 8;
		sum += addr;
		type = get_data(p, 2); p += 2;
		//sum += type;
		out = data + addr;
		if (type == 0) {
		    for (i=0; i<len; i++) {
			if (out < data || out >= data + size) {
				fprintf(stderr, "range error 0x%0x \n",out - data);
				ret = -1;
				goto err;
			}
			r = get_data(p, 2); p += 2;
			*out++ = r;
			sum += r;
			ret ++;
		    }
		} else {
			continue;
		}
		fsum = get_data(p, 2); p += 2;
		sum += fsum;
		if ((sum & 0xff) != 0 ) {
			fprintf(stderr, "checksum error :%d addr 0x%0x fsum %02x result %02x\n",line_no, addr,fsum,sum & 0xff);
			//ret = -1;
			//goto err;
		}
		if (type == 1) break;
	}
err:
	fclose(fp);
	return ret;
}

int save_ihex(char *fname, char *data, int size) {
	FILE *fp;
	char buf[256];
	int i,r,l,addr = 0;
	int type,sum,pos;
	fp = fopen(fname, "w");
	if (!fp) return -1;
	while (size > 0) {
		pos = 0;
		type = 0;
		sum = 0;
		l = size;
		if (l > 0x10) l = 0x10;
		pos += sprintf(buf+pos,":%02x%04x%02x",l,addr,type);
		sum = l + addr + (addr >> 8) + type;
		for (i=0; i < l; i++) {
			r = data[addr + i] & 0xff;
			sum += r;
			pos += sprintf(buf+pos,"%02x",r);
		}
		pos += sprintf(buf+pos,"%02x\r\n", -sum & 0xff);
		for (i=0; i< pos; i++) {
			if ('a' <= buf[i] && buf[i] <= 'f') {
				buf[i] = buf[i] - 'a' + 'A';
			}
		}
		fputs(buf, fp);
		size -= l;
		addr += l;
	}
	fprintf(fp,":00000001FF\r\n");
	fclose(fp);
	return 0;
}

#ifdef LOCAL_TEST
main(int argc,char *argv[]) {
	int r;
	char data[1024*16];
	memset(data, 0xff , sizeof(data));
	r = load_ihex(argv[1], data, 1024 * 16);
	printf("load_ihex %s = %d\n",argv[1], r);
	save_ihex("tmp.hex", data, 1024 * 16);
}
#endif
